@@ -1,11 +1,21 @@
 package Net::SFTP::Foreign::Common;
 
-our $VERSION = '1.66_01';
+our $VERSION = '1.68_01';
 
 use strict;
 use warnings;
 use Carp;
-use Scalar::Util qw(dualvar tainted);
+
+BEGIN {
+    # Some versions of Scalar::Util are crippled
+    require Scalar::Util;
+    eval { Scalar::Util->import(qw(dualvar tainted)); 1 }
+        or do {
+            *tainted = sub { croak "The version of Scalar::Util installed on your system "
+                                 . "does not provide 'tainted'" };
+            *dualvar = sub { $_[0] };
+        };
+}
 
 use Net::SFTP::Foreign::Helpers qw(_gen_wanted _ensure_list _debug _glob_to_regex _is_lnk _is_dir $debug);
 use Net::SFTP::Foreign::Constants qw(:status);
@@ -85,6 +95,13 @@ sub die_on_error {
     $sftp->{_error} and croak(@_ ? "@_: $sftp->{_error}" : $sftp->{_error});
 }
 
+sub _ok_or_autodie {
+    my $sftp = shift;
+    return 1 unless $sftp->{_error};
+    $sftp->{_autodie} and croak $sftp->{_error};
+    undef;
+}
+
 sub _set_errno {
     my $sftp = shift;
     if ($sftp->{_error}) {
@@ -111,6 +128,15 @@ sub _set_errno {
     }
 }
 
+sub _best_effort {
+    my $sftp = shift;
+    my $best_effort = shift;
+    my $method = shift;
+    local ($sftp->{_error}, $sftp->{_autodie}) if $best_effort;
+    $sftp->$method(@_);
+    return (($best_effort or not $sftp->{_error}) ? 1 : undef);
+}
+
 sub _call_on_error {
     my ($sftp, $on_error, $entry) = @_;
     $on_error and $sftp->error
@@ -378,8 +404,29 @@ sub glob {
 
 sub test_d {
     my ($sftp, $name) = @_;
-    my $a = $sftp->stat($name);
-    $a ? _is_dir($a->perm) : undef;
+    {
+        local $sftp->{_autodie};
+        my $a = $sftp->stat($name);
+        return _is_dir($a->perm) if $a;
+    }
+    if ($sftp->{_status} == SSH2_FX_NO_SUCH_FILE) {
+        $sftp->_clear_error_and_status;
+        return undef;
+    }
+    $sftp->_ok_or_autodie;
+}
+
+sub test_e {
+    my ($sftp, $name) = @_;
+    {
+        local $sftp->{_autodie};
+        $sftp->stat($name) and return 1;
+    }
+    if ($sftp->{_status} == SSH2_FX_NO_SUCH_FILE) {
+        $sftp->_clear_error_and_status;
+        return undef;
+    }
+    $sftp->_ok_or_autodie;
 }
 
 1;
